/*
 * Lua Commander - Lua Notepad
 *
 * Copyright (C) 1999-2000, Antonio E. Scuri, Tecgraf/PUC-Rio.
 */


#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#include <iup.h>
#include <iupkey.h>
#include <lua.h>

#include "lnotepad.h"
#include "lconsole.h"
#include "utl.h"


static struct _luaNotepad
{
  Ihandle *iupDialog,
          *lbLineNumber,
          *mlText;
  int close,      /* return value of the Close callback */
      line_num;   /* last line number updated */
}notepad;


int luaNotepadClear(void)
{
  IupSetAttribute(notepad.mlText, IUP_VALUE, "");
  return IUP_DEFAULT;
}

static void NotepadUpdateLine(int lin)
{
  if (notepad.line_num != lin)
  {
    IupSetfAttribute(notepad.lbLineNumber, IUP_TITLE, "%d", lin);
    notepad.line_num = lin;
  }
}

static void NotepadUpdateLineNum(void)
{
  int col, lin;
  sscanf(IupGetAttribute(notepad.mlText, IUP_CARET), "%d,%d", &lin, &col);
  NotepadUpdateLine(lin);
}

static int NotepadNotify(Ihandle* self, int c)
{
  int col, lin, num_lin;
  sscanf(IupGetAttribute(notepad.mlText, IUP_CARET), "%d,%d", &lin, &col);

  if (c == K_UP)
    lin--;
  if (c == K_DOWN)
    lin++;

  if (lin < 1)
    lin = 1;

  num_lin = utlStrLines(IupGetAttribute(notepad.mlText, IUP_VALUE));
  if (lin > num_lin)
    lin = num_lin;

  NotepadUpdateLine(lin);

  return IUP_DEFAULT;
}

void luaNotepadLoad(char* file_name)
{
  char *text = utlLoadTextFile(file_name);
  if (!text)
  {
    IupMessage("Error!", "Can not load file.");
    return;
  }

  IupStoreAttribute(notepad.mlText, IUP_VALUE, text);
  NotepadUpdateLineNum();

  free(text);
}

static int NotepadClose(void)
{
  return notepad.close;
}

void luaNotepadSave(char* file_name)
{
  if (!utlSaveTextFile(file_name, luaNotepadGet()))
    IupMessage("Error!", "Can not save file.");
}

static char luaNotepadFileName[1024] = "*.lua";

int luaNotepadLoadDlg(void)
{
  if (!utlGetLuaFile(luaNotepadFileName, 0))
    return IUP_DEFAULT;

  luaNotepadLoad(luaNotepadFileName);
  return IUP_DEFAULT;
}

int luaNotepadSaveDlg(void)
{
  if (!utlGetLuaFile(luaNotepadFileName, 1))
    return IUP_DEFAULT;

  luaNotepadSave(luaNotepadFileName);
  return IUP_DEFAULT;
}

char* luaNotepadGet(void)
{
  return IupGetAttribute(notepad.mlText, IUP_VALUE);
}

void luaNotepadSetf(char* format, ...)
{
  char value[65535];
  va_list arglist;
  va_start(arglist, format);
  vsprintf(value, format, arglist);
  IupStoreAttribute(notepad.mlText, IUP_VALUE, value);
  NotepadUpdateLineNum();
}

void luaNotepadAddf(char* format, ...)
{
  char value[65535];
  va_list arglist;
  va_start(arglist, format);
  vsprintf(value, format, arglist);
  IupSetAttribute(notepad.mlText, IUP_CARET, "65535,65535");
  IupSetAttribute(notepad.mlText, IUP_APPEND, value);
  NotepadUpdateLineNum();
}

void luaNotepadInsertf(char* format, ...)
{
  char value[65535];
  va_list arglist;
  va_start(arglist, format);
  vsprintf(value, format, arglist);
  IupSetAttribute(notepad.mlText, IUP_APPEND, value);
  NotepadUpdateLineNum();
}

void luaNotepadSet(char* text)
{
  IupStoreAttribute(notepad.mlText, IUP_VALUE, text);
  NotepadUpdateLineNum();
}

void luaNotepadAdd(char* text)
{
  IupSetAttribute(notepad.mlText, IUP_CARET, "65535,65535");
  IupSetAttribute(notepad.mlText, IUP_APPEND, text);
  NotepadUpdateLineNum();
}

void luaNotepadInsert(char* text)
{
  IupSetAttribute(notepad.mlText, IUP_APPEND, text);
  NotepadUpdateLineNum();
}

int luaNotepadShow(void)
{
  IupShow(notepad.iupDialog);
  return IUP_DEFAULT;
}

static int NotepadRun(void)
{
  char* value = IupGetAttribute(notepad.mlText, IUP_VALUE);

  if (value && value[0] != 0)
  {
    char* buffer = utlStrDup(value);
    luaConsoleEnterCommandStr("dostring(lnpadGet())");
    lua_dostring(buffer);
    free(buffer);
  }
  
  return IUP_DEFAULT;
}

static void NotepadGet(void)
{
  char* value = IupGetAttribute(notepad.mlText, IUP_VALUE);
  lua_pushstring(value);
}

static void NotepadSet(void)
{
  char* text;
  text = lua_getstring(lua_getparam(1));
  if (text) luaNotepadSet(text);
}

static void NotepadAdd(void)
{
  char* text;
  text = lua_getstring(lua_getparam(1));
  if (text) luaNotepadAdd(text);
}

static void NotepadInsert(void)
{
  char* text;
  text = lua_getstring(lua_getparam(1));
  if (text) luaNotepadInsert(text);
}

static void NotepadLoad(void)
{
  char* text;
  text = lua_getstring(lua_getparam(1));
  if (text) luaNotepadLoad(text);
}

static void NotepadSave(void)
{
  char* text;
  text = lua_getstring(lua_getparam(1));
  if (text) luaNotepadSave(text);
}

void luaNotepadCreate(char* parent, int close)
{
  Ihandle *vbLabels, *hbToolBar;

  memset(&notepad, 0, sizeof(notepad));

  IupSetFunction("luaNotepadSaveDlg", (Icallback)luaNotepadSaveDlg);
  IupSetFunction("luaNotepadLoadDlg", (Icallback)luaNotepadLoadDlg);
  IupSetFunction("luaNotepadClear", (Icallback)luaNotepadClear);
  IupSetFunction("luaNotepadClose", (Icallback)NotepadClose);
  IupSetFunction("luaNotepadNotify", (Icallback)NotepadNotify);
  IupSetFunction("luaNotepadRun", (Icallback)NotepadRun);

  hbToolBar = IupHbox(
    IupSetAttributes(IupLabel(""),"IMAGE=LuaButton"),
    IupSetAttributes(IupFill(), "EXPAND=NO, SIZE=10"),
    IupSetAttributes(IupButton("", "luaNotepadClear"), "TIP=\"Clears the text contents.  Ctrl+Del\", IMAGE=utlNewButton"),
    IupSetAttributes(IupButton("", "luaNotepadLoadDlg"), "TIP=\"Loads a text file from disk.\", IMAGE=utlOpenButton"),
    IupSetAttributes(IupButton("", "luaNotepadSaveDlg"), "TIP=\"Saves the text to disk.\", IMAGE=utlSaveButton"),
    IupSetAttributes(IupFill(), "EXPAND=NO, SIZE=20"),
    IupSetAttributes(IupButton("Do String", "luaNotepadRun"), "TIP=\"Executes the LUA code in the notepad. Ctrl+R\", SIZE=45"),
    IupFill(), 
    IupLabel("Line:"),
    IupSetAttributes(notepad.lbLineNumber = IupLabel(""), "SIZE=30"),
    NULL);

  IupSetAttributes(hbToolBar,"MARGIN=0x0, GAP=0, ALIGNMENT=ACENTER");

  vbLabels = IupVbox(
    hbToolBar,
    notepad.mlText = IupSetAttributes(IupMultiLine("luaNotepadNotify"), "EXPAND=YES"),
    NULL);

  IupSetAttributes(vbLabels,"MARGIN=5x5, GAP=5");
  IupSetAttribute(notepad.mlText, IUP_FONT, IUP_COURIER_NORMAL_12);
  IupSetAttribute(notepad.mlText, IUP_K_ANY, "luaNotepadNotify");

  notepad.iupDialog = IupDialog(vbLabels);
  if (parent) IupSetAttribute(notepad.iupDialog,IUP_PARENTDIALOG, parent);
  IupSetAttribute(notepad.iupDialog,IUP_CLOSE_CB,"luaNotepadClose");
  IupSetAttribute(notepad.iupDialog,IUP_TITLE, "Lua Notepad");
  IupSetAttribute(notepad.iupDialog,IUP_K_cDEL, "luaNotepadClear");
  IupSetAttribute(notepad.iupDialog,IUP_K_cR, "luaNotepadRun");
                  
  IupSetHandle("luaNotepad", notepad.iupDialog);
  IupSetAttribute(notepad.iupDialog,IUP_SIZE, "300x200");
  IupSetAttribute(notepad.iupDialog,IUP_ICON, "LuaIcon");

  lua_register("lnShow", (lua_CFunction)luaNotepadShow);
  lua_register("lnpadGet", NotepadGet);
  lua_register("lnpadClear", (lua_CFunction)luaNotepadClear);
  lua_register("lnpadSet", NotepadSet);
  lua_register("lnpadAdd", NotepadAdd);
  lua_register("lnpadInsert", NotepadInsert);
  lua_register("lnpadLoad", NotepadLoad);
  lua_register("lnpadSave", NotepadSave);

  notepad.close = close;
  notepad.line_num = 0;

  IupShowXY(notepad.iupDialog, 270, 220);
}

void luaNotepadKill(void)
{
  IupDestroy(notepad.iupDialog);
}

